@@ -1,4 +1,4 @@
-# Copyright 2015 IBM Corp.
+# Copyright 2015, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -19,13 +19,45 @@ import mock
 
 from ceilometer.compute.virt import inspector as virt_inspector
 from oslotest import base
-from pypowervm.tasks.monitor import lpar as pvm_lpar_stat
+from pypowervm.helpers import log_helper as log_hlp
+from pypowervm.helpers import vios_busy as vio_hlp
 from pypowervm.tests import test_fixtures as api_fx
 
 from ceilometer_powervm.compute.virt.powervm import inspector as p_inspect
 from ceilometer_powervm.tests.compute.virt.powervm import pvm_fixtures
 
 
+class TestPowerVMInspectorInit(base.BaseTestCase):
+    """Tests the initialization of the VM Inspector."""
+
+    @mock.patch('pypowervm.tasks.monitor.util.LparMetricCache')
+    @mock.patch('pypowervm.tasks.monitor.util.ensure_ltm_monitors')
+    @mock.patch('ceilometer_powervm.compute.virt.powervm.inspector.'
+                'PowerVMInspector._get_host_uuid')
+    @mock.patch('pypowervm.adapter.Adapter')
+    @mock.patch('pypowervm.adapter.Session')
+    def test_init(self, mock_session, mock_adapter, mock_get_host_uuid,
+                  mock_ensure_ltm, mock_cache):
+        # Mock up data
+        mock_get_host_uuid.return_value = 'host_uuid'
+
+        # Invoke
+        inspector = p_inspect.PowerVMInspector(None)
+
+        # Validate
+        mock_session.assert_called_once_with(conn_tries=300)
+        mock_adapter.assert_called_once_with(
+            mock_session.return_value,
+            helpers=[log_hlp.log_helper, vio_hlp.vios_busy_retry_helper])
+
+        mock_get_host_uuid.assert_called_once_with(mock_adapter.return_value)
+        mock_ensure_ltm.assert_called_once_with(mock_adapter.return_value,
+                                                'host_uuid')
+        mock_cache.assert_called_once_with(mock_adapter.return_value,
+                                           'host_uuid')
+        self.assertEqual(mock_cache.return_value, inspector.vm_metrics)
+
+
 class TestPowerVMInspector(base.BaseTestCase):
 
     def setUp(self):
@@ -44,44 +76,21 @@ class TestPowerVMInspector(base.BaseTestCase):
         with mock.patch('ceilometer_powervm.compute.virt.powervm.inspector.'
                         'PowerVMInspector._get_host_uuid'):
             # Create the inspector
-            self.inspector = p_inspect.PowerVMInspector()
+            self.inspector = p_inspect.PowerVMInspector(None)
             self.inspector.vm_metrics = self.mock_metrics
 
-    def test_inspect_cpus(self):
-        """Validates PowerVM's inspect_cpus method."""
+    def test_inspect_instance_stats(self):
+        """Validates PowerVM's inspect_instance method."""
         # Validate that an error is raised if the instance can't be found
         # in the sample
         self.mock_metrics.get_latest_metric.return_value = None, None
         self.assertRaises(virt_inspector.InstanceNotFoundException,
-                          self.inspector.inspect_cpus, mock.Mock())
-
-        # Build a response from the metric cache.
-        metric = mock.MagicMock()
-        metric.processor.util_cap_proc_cycles = 5
-        metric.processor.util_uncap_proc_cycles = 6
-        metric.processor.virt_procs = 12
-        self.mock_metrics.get_latest_metric.return_value = mock.Mock(), metric
-
-        # Invoke with the test data
-        resp = self.inspector.inspect_cpus(mock.Mock())
-
-        # Validate the metrics that came back
-        self.assertEqual(11, resp.time)
-        self.assertEqual(12, resp.number)
-
-    def test_inspect_cpu_util(self):
-        """Validates PowerVM's inspect_cpu_util method."""
-        # Validate that an error is raised if the instance can't be found in
-        # the sample data.
-        self.mock_metrics.get_latest_metric.return_value = None, None
-        self.mock_metrics.get_previous_metric.return_value = None, None
-        self.assertRaises(virt_inspector.InstanceNotFoundException,
-                          self.inspector.inspect_cpu_util,
-                          mock.Mock(), duration=30)
+                          self.inspector.inspect_instance, mock.Mock())
 
         def mock_metric(util_cap, util_uncap, idle, donated, entitled):
             """Helper method to create mock proc metrics."""
             metric = mock.MagicMock()
+            metric.processor.virt_procs = 12
             metric.processor.util_cap_proc_cycles = util_cap
             metric.processor.util_uncap_proc_cycles = util_uncap
             metric.processor.idle_proc_cycles = idle
@@ -94,8 +103,9 @@ class TestPowerVMInspector(base.BaseTestCase):
         self.mock_metrics.get_latest_metric.return_value = (
             mock.Mock(), mock_metric(7000, 50, 1000, 5000, 10000))
         self.mock_metrics.get_previous_metric.return_value = None, None
-        self.assertRaises(virt_inspector.InstanceNotFoundException,
-                          self.inspector.inspect_cpu_util, mock.Mock())
+        resp = self.inspector.inspect_instance(mock.Mock())
+        self.assertEqual(7050, resp.cpu_time)
+        self.assertEqual(12, resp.cpu_number)
 
         # Mock up a mixed use environment.
         cur = mock_metric(7000, 50, 1000, 5000, 10000)
@@ -104,8 +114,8 @@ class TestPowerVMInspector(base.BaseTestCase):
         self.mock_metrics.get_previous_metric.return_value = mock.Mock(), prev
 
         # Execute and validate
-        resp = self.inspector.inspect_cpu_util(mock.Mock())
-        self.assertEqual(.5, resp.util)
+        resp = self.inspector.inspect_instance(mock.Mock())
+        self.assertEqual(.5, resp.cpu_util)
 
         # Mock an instance with a dedicated processor, but idling and donating
         # cycles to others.  In these scenarios, util_cap shows full use, but
@@ -116,8 +126,8 @@ class TestPowerVMInspector(base.BaseTestCase):
         self.mock_metrics.get_previous_metric.return_value = mock.Mock(), prev
 
         # Execute and validate
-        resp = self.inspector.inspect_cpu_util(mock.Mock())
-        self.assertEqual(51.0, resp.util)
+        resp = self.inspector.inspect_instance(mock.Mock())
+        self.assertEqual(51.0, resp.cpu_util)
 
         # Mock an instance with a shared processor.  By nature, this doesn't
         # idle or donate.  If it is 'idling' it is simply giving the cycles
@@ -128,8 +138,8 @@ class TestPowerVMInspector(base.BaseTestCase):
         self.mock_metrics.get_previous_metric.return_value = mock.Mock(), prev
 
         # Execute and validate
-        resp = self.inspector.inspect_cpu_util(mock.Mock())
-        self.assertEqual(80.0, resp.util)
+        resp = self.inspector.inspect_instance(mock.Mock())
+        self.assertEqual(80.0, resp.cpu_util)
 
         # Mock an instance with a shared processor - but using cycles from
         # the uncap pool.  This means it is using extra cycles from other
@@ -141,8 +151,8 @@ class TestPowerVMInspector(base.BaseTestCase):
 
         # Execute and validate.  This should be running at 300% CPU
         # utilization.  Fast!
-        resp = self.inspector.inspect_cpu_util(mock.Mock())
-        self.assertEqual(300.0, resp.util)
+        resp = self.inspector.inspect_instance(mock.Mock())
+        self.assertEqual(300.0, resp.cpu_util)
 
         # Mock an instance that hasn't been started yet.
         cur = mock_metric(0, 0, 0, 0, 0)
@@ -151,22 +161,22 @@ class TestPowerVMInspector(base.BaseTestCase):
         self.mock_metrics.get_previous_metric.return_value = mock.Mock(), prev
 
         # This should have 0% utilization
-        resp = self.inspector.inspect_cpu_util(mock.Mock())
-        self.assertEqual(0.0, resp.util)
+        resp = self.inspector.inspect_instance(mock.Mock())
+        self.assertEqual(0.0, resp.cpu_util)
 
     @staticmethod
-    def _mock_vnic_metric(rec_bytes, tx_bytes, rec_pkts, tx_pkts, phys_loc):
+    def _mock_vnic_metric(rec_bytes, tx_bytes, rec_pkts, tx_pkts, phys_loc,
+                          drop_pkts):
         """Helper method to create a specific mock network metric."""
-        return pvm_lpar_stat.LparCNA(mock.Mock(
-            received_bytes=rec_bytes, sent_bytes=tx_bytes,
-            received_packets=rec_pkts, sent_packets=tx_pkts,
-            physical_location=phys_loc))
+        return mock.Mock(received_bytes=rec_bytes, sent_bytes=tx_bytes,
+                         received_packets=rec_pkts, sent_packets=tx_pkts,
+                         dropped_packets=drop_pkts, physical_location=phys_loc)
 
     def _build_cur_mock_vnic_metrics(self):
         """Helper method to create mock network metrics."""
-        cna1 = self._mock_vnic_metric(1000, 1000, 10, 10, 'a')
-        cna2 = self._mock_vnic_metric(2000, 2000, 20, 20, 'b')
-        cna3 = self._mock_vnic_metric(3000, 3000, 30, 30, 'c')
+        cna1 = self._mock_vnic_metric(1000, 1000, 10, 10, 'a', 1)
+        cna2 = self._mock_vnic_metric(2000, 2000, 20, 20, 'b', 2)
+        cna3 = self._mock_vnic_metric(3000, 3000, 30, 30, 'c', 3)
 
         metric = mock.MagicMock()
         metric.network.cnas = [cna1, cna2, cna3]
@@ -174,8 +184,8 @@ class TestPowerVMInspector(base.BaseTestCase):
 
     def _build_prev_mock_vnic_metrics(self):
         """Helper method to create mock network metrics."""
-        cna1 = self._mock_vnic_metric(1000, 1000, 10, 10, 'a')
-        cna2 = self._mock_vnic_metric(200, 200, 20, 20, 'b')
+        cna1 = self._mock_vnic_metric(1000, 1000, 10, 10, 'a', 1)
+        cna2 = self._mock_vnic_metric(200, 200, 20, 20, 'b', 2)
 
         metric = mock.MagicMock()
         metric.network.cnas = [cna1, cna2]
@@ -220,13 +230,17 @@ class TestPowerVMInspector(base.BaseTestCase):
         resp = list(self.inspector.inspect_vnics(mock.Mock()))
         self.assertEqual(3, len(resp))
 
-        interface1, stats1 = resp[0]
-        self.assertEqual('aa:bb:cc:dd:ee:ff', interface1.mac)
-        self.assertEqual('a', interface1.name)
+        stats1 = resp[0]
+        self.assertEqual('aa:bb:cc:dd:ee:ff', stats1.mac)
+        self.assertEqual('a', stats1.name)
         self.assertEqual(1000, stats1.rx_bytes)
         self.assertEqual(1000, stats1.tx_bytes)
         self.assertEqual(10, stats1.rx_packets)
         self.assertEqual(10, stats1.tx_packets)
+        self.assertEqual(1, stats1.rx_drop)
+        self.assertEqual(0, stats1.tx_drop)
+        self.assertEqual(0, stats1.rx_errors)
+        self.assertEqual(0, stats1.tx_errors)
 
     @mock.patch('pypowervm.wrappers.network.CNA.wrap')
     def test_inspect_vnic_rates(self, mock_wrap):
@@ -266,23 +280,23 @@ class TestPowerVMInspector(base.BaseTestCase):
         self.assertEqual(3, len(resp))
 
         # First metric.  No delta
-        interface1, stats1 = resp[0]
-        self.assertEqual('aa:bb:cc:dd:ee:ff', interface1.mac)
-        self.assertEqual('a', interface1.name)
+        stats1 = resp[0]
+        self.assertEqual('aa:bb:cc:dd:ee:ff', stats1.mac)
+        self.assertEqual('a', stats1.name)
         self.assertEqual(0, stats1.rx_bytes_rate)
         self.assertEqual(0, stats1.tx_bytes_rate)
 
         # Second metric
-        interface2, stats2 = resp[1]
-        self.assertEqual('aa:bb:cc:dd:ee:11', interface2.mac)
-        self.assertEqual('b', interface2.name)
+        stats2 = resp[1]
+        self.assertEqual('aa:bb:cc:dd:ee:11', stats2.mac)
+        self.assertEqual('b', stats2.name)
         self.assertEqual(60.0, stats2.rx_bytes_rate)
         self.assertEqual(60.0, stats2.tx_bytes_rate)
 
         # Third metric had no previous.
-        interface3, stats3 = resp[2]
-        self.assertEqual('aa:bb:cc:dd:ee:22', interface3.mac)
-        self.assertEqual('c', interface3.name)
+        stats3 = resp[2]
+        self.assertEqual('aa:bb:cc:dd:ee:22', stats3.mac)
+        self.assertEqual('c', stats3.name)
         self.assertEqual(100.0, stats3.rx_bytes_rate)
         self.assertEqual(100.0, stats3.tx_bytes_rate)
 
@@ -294,7 +308,7 @@ class TestPowerVMInspector(base.BaseTestCase):
                       read_bytes=read_bytes, write_bytes=write_bytes)
         # Have to do this method as name is a special field.
         m.configure_mock(name=name)
-        return pvm_lpar_stat.LparVirtStorageAdpt(m)
+        return m
 
     def _build_cur_mock_stor_metrics(self):
         """Helper method to create mock storage metrics."""
@@ -351,17 +365,17 @@ class TestPowerVMInspector(base.BaseTestCase):
         self.assertEqual(3, len(resp))
 
         # Two vSCSI's
-        disk1, stats1 = resp[0]
-        self.assertEqual('vscsi1', disk1.device)
+        stats1 = resp[0]
+        self.assertEqual('vscsi1', stats1.device)
         self.assertEqual(33, stats1.iops_count)
 
-        disk2, stats2 = resp[1]
-        self.assertEqual('vscsi2', disk2.device)
+        stats2 = resp[1]
+        self.assertEqual('vscsi2', stats2.device)
         self.assertEqual(133, stats2.iops_count)
 
         # Next is the vFC metric
-        disk3, stats3 = resp[2]
-        self.assertEqual('vfc1', disk3.device)
+        stats3 = resp[2]
+        self.assertEqual('vfc1', stats3.device)
         self.assertEqual(66, stats3.iops_count)
 
     def test_inspect_disks(self):
@@ -389,27 +403,33 @@ class TestPowerVMInspector(base.BaseTestCase):
         self.assertEqual(3, len(resp))
 
         # Two vSCSIs.
-        disk1, stats1 = resp[0]
-        self.assertEqual('vscsi1', disk1.device)
+        stats1 = resp[0]
+        self.assertEqual('vscsi1', stats1.device)
         self.assertEqual(1000, stats1.read_requests)
         self.assertEqual(100000, stats1.read_bytes)
         self.assertEqual(1000, stats1.write_requests)
         self.assertEqual(100000, stats1.write_bytes)
         self.assertEqual(0, stats1.errors)
+        self.assertEqual(0, stats1.wr_total_times)
+        self.assertEqual(0, stats1.rd_total_times)
 
-        disk2, stats2 = resp[1]
-        self.assertEqual('vscsi2', disk2.device)
+        stats2 = resp[1]
+        self.assertEqual('vscsi2', stats2.device)
         self.assertEqual(2000, stats2.read_requests)
         self.assertEqual(200000, stats2.read_bytes)
         self.assertEqual(2000, stats2.write_requests)
         self.assertEqual(200000, stats2.write_bytes)
-        self.assertEqual(0, stats1.errors)
+        self.assertEqual(0, stats2.errors)
+        self.assertEqual(0, stats2.wr_total_times)
+        self.assertEqual(0, stats2.rd_total_times)
 
         # Next is the vFC metric
-        disk3, stats3 = resp[2]
-        self.assertEqual('vfc1', disk3.device)
+        stats3 = resp[2]
+        self.assertEqual('vfc1', stats3.device)
         self.assertEqual(3000, stats3.read_requests)
         self.assertEqual(300000, stats3.read_bytes)
         self.assertEqual(3000, stats3.write_requests)
         self.assertEqual(300000, stats3.write_bytes)
         self.assertEqual(0, stats3.errors)
+        self.assertEqual(0, stats3.wr_total_times)
+        self.assertEqual(0, stats3.rd_total_times)
